clear all
clc
close all

DBPath  = 'C:\Users\creasger\Dropbox\ResearchProjects\RealizedBetaGARCH\JAEpublish\RepCode\';
WPath   = 'Estimate';
addpath(genpath([DBPath WPath]));
DataPath = [DBPath 'Data-v1\'];

cd([DBPath WPath]);

parNames = {'Permno','log(h0)','mu0','a','b','c','d','tau1','tau2','xi','del1','del2','rho0','at','bt','ct','psi','phi'};

options = optimset('TolX',1e-4,'TolFun',1e-4,'Algorithm','sqp','MaxFunEvals',10000,'Display','off'); 

% Init setting and select what to do	    
cd(DataPath);

fid  = fopen('headers.txt','r'); tmp = textscan(fid,'%s', 'delimiter', '$'); fclose(fid);
VarNm = tmp{1};

useTrim = [0.0015 0.9985];
time1   = tic;

useData = 'CVXdata.txt'; 
fprintf('Using: %s\n',useData);
useData = dlmread([DataPath 'CVXdata.txt']);

useData(useData(:,1)<20020000,:) = [];
useData(diff(useData(:,16:17),1,2)<5*60*60,:) = [];

cd([DBPath WPath]);

mtmp = useData;

stS = length(mtmp(mtmp(:,1)<20020000,1)) +2;
enS = length(mtmp(mtmp(:,1)<20100000,1)); 
fprintf('Estimation period: %9.8g - %8.8g\n',mtmp(stS,1),mtmp(enS,1));

rNm = {'retSPY';'retCVX'; 'RCorr'};
VarNm(21:23) = rNm;
mtmp(2:end,21:22) = diff(100*log(abs(mtmp(:,ismember(VarNm, {'padjA.close','padjB.close'})))),1);
g_mRK = mtmp(:,ismember(VarNm, {'CVM202','CVM101','CVM201'})); g_mRK = g_mRK(:,[3 1 2]);
mtmp(:,23) = g_mRK(:,3)./sqrt(bsxfun(@times,g_mRK(:,1),g_mRK(:,2)));

%*** Add risk free rate

FFhead = {'DATE','Rf'};
FFact  = dlmread([DataPath 'FFdata.txt']);
cFirst = size(mtmp,2); 
mtmp   = [mtmp zeros(size(mtmp,1),2)];
for j = 1:size(mtmp,1), mtmp(j,cFirst+1:end) = 100*FFact(FFact(:,1)==mtmp(j,1),2:end); end;
VarNm  = [VarNm; FFhead(2:end)'];

%*** Make excess returns

mtmp(:,ismember(VarNm, {'retSPY'})) = mtmp(:,ismember(VarNm, {'retSPY'}))-mtmp(:,ismember(VarNm, {'Rf'}));
mtmp(:,ismember(VarNm, 'retCVX' )) = mtmp(:,ismember(VarNm, 'retCVX'))-mtmp(:,ismember(VarNm, {'Rf'}));

% --------------------------------------------------------------------------------
% --------------------------------------------------------------------------------
% --------------------------------------------------------------------------------
% --------------------------------------------------------------------------------

g_mY  = mtmp(:,ismember(VarNm, 'retSPY'));  % get Market returns
g_mRK = mtmp(:,ismember(VarNm, 'RKspy')); % get Market Realized Variance
g_mY  = g_mY(stS:enS); g_mRK = g_mRK(stS:enS);

par  = [0.1 0.0 0.1 0.6 0.3 0.01 0.1];	par = [par  -0.5 0.05 0.05];

p0 = par;
[par_A,fval,exitflag,output,lambda,grad,~] = fmincon(@(p) fLogL_Vo2(p,g_mY,g_mRK),p0,...
 [],[],[],[],-10*ones(length(par),1),Inf*ones(length(par),1),[],options); 


Hess = hessian_2sided(@fLogL_Vo2,par_A',g_mY,g_mRK);

stder  = sqrt(diag(inv(Hess)));
t_stat = bsxfun(@rdivide,par_A',stder);
res_A  = [par_A' stder t_stat 2*(1- cdf('Normal',abs(t_stat),0,1))];

% --------------------------------------------------------------------------------

model = TrimByC(mtmp(stS:enS,:),22,useTrim);
stS = 1; enS = size(model,1);

g_mY  = model(:,ismember(VarNm, rNm{1}));  % get Market returns
g_mRK = model(:,ismember(VarNm, 'RKspy')); % get Market Realized Variance
g_mY  = g_mY(stS:enS); g_mRK = g_mRK(stS:enS);

[mLikSpy,g_mH] =  fLogLikc(par_A,g_mY,g_mRK);
fprintf('1st done!\n');

% --------------------------------------------------------------------------------

g_mY  = model(:,ismember(VarNm, rNm{2}));  % get Market returns
g_mRK = model(:,ismember(VarNm, 'CVM202')); % get Market Realized Variance
g_mY  = g_mY(stS:enS); g_mRK = g_mRK(stS:enS);

p0 = par_A;
[par_B,fval,exitflag,output,lambda,grad,~] = fmincon(@(p) fLogL_Vo2(p,g_mY,g_mRK),p0,...
 [],[],[],[],-10*ones(length(par_A),1),Inf*ones(length(par_A),1),[],options); 

fprintf('2nd done!\n');

[~,g_mh] =  fLogLikc(par_B,g_mY,g_mRK);

% --------------------------------------------------------------------------------

g_mY  = model(:,ismember(VarNm, rNm{2}));  % get Individual asset returns
g_mRK = model(:,ismember(VarNm, {'CVM202','CVM101','CVM201'})); % get Realized CoVar
g_mRK = g_mRK(:,[3 1 2]); g_mY  = g_mY(stS:enS,:); g_mRK = g_mRK(stS:enS,:);

corr     = g_mRK(:,3)./sqrt(g_mRK(:,1).*g_mRK(:,2));
mean_rho = mean(corr); 

par  = [Fisher(mean_rho) 0.1 0.6 0.3 0.1 1];

p0 = par;
[par_C,fval,exitflag,output,lambda,grad,~] = fmincon(@(p) fLogL_Joint1(p,g_mY,g_mRK,corr,g_mh,g_mH),p0,...
 [],[],[],[],-10*ones(length(par),1),Inf*ones(length(par),1),[],options); 

% --------------------------------------------------------------------------------
% --------------------------------------------------------------------------------

p0 = [par_B(1:5) 0.0 par_B(6:end) par_C];

if abs(sum(p0(4:5)))>=1.0, p0(4:5) = p0(4:5)*0.95; end;

[par_J,fval,exitflag,output,lambda,grad,~] = fmincon(@(p) fLogL_Joint2(p,g_mY,g_mRK,corr,g_mH),p0,...
 [],[],[],[],-10*ones(length(par_C),1),Inf*ones(length(par_C),1),[],options); 

[~,g_mRhot, g_mVo] =  fLogL_Joint2(par_J,g_mY,g_mRK,corr,g_mH);

Hess   = hessian_2sided(@fLogL_Joint2,par_J',g_mY,g_mRK,corr,g_mH);
stder  = sqrt(diag(inv(Hess)));
t_stat = bsxfun(@rdivide,par_J',stder);
res    = [par_J' stder t_stat 2*(1- cdf('Normal',abs(t_stat),0,1))];

fprintf('3rd done!\n');

% --------------------------------------------------------------------------------
% --------------------------------------------------------------------------------

fprintf('\nMarket model is CVX on SPY\n');
rowinf = {'%-10s',{'log(h0)','mu0','a','b','c','tau1','tau2','xi','del1','del2'}};
colinf = {'%10s',{'Parm','StdErr','t-stat','p-vals'},{'%10.4g','%10.4f','%10.4f','%10.4f'}};

PrintTable(rowinf,colinf,res_A);
fprintf('\nLog-likelihood at max: %16.5f\n\n', mLikSpy);

rowinf = {'%-10s',{'log(h0)','mu0','a','b','c','d','tau1','tau2','xi','del1','del2'}};
colinf = {'%10s',{'Parm','StdErr','t-stat','p-vals'},{'%10.4g','%10.4f','%10.4f','%10.4f'}};

PrintTable(rowinf,colinf,res(1:11,:));
fprintf('\n');

rowinf = {'%-10s',{'rho0','at','bt','ct','psi','phi'}};
colinf = {'%10s',{'Parm','StdErr','t-stat','p-vals'},{'%10.4g','%10.4f','%10.4f','%10.4f'}};

PrintTable(rowinf,colinf,res(12:end,:));
fprintf('\n');

E = [g_mH(:,2) g_mVo(:,2) g_mRhot(:,2)];
Sigma = (E'*E)/size(E,1);
fprintf('Sigma matrix:  %10.4f\n', Sigma(1,1));
fprintf('               %10.4f %10.4f\n', Sigma(2,1:2));
fprintf('               %10.4f %10.4f %10.4f\n\n', Sigma(3,1:3));

fprintf('Log-likelihood at max:      %16.5f\n',fval);
fprintf('Full Log-likelihood at max: %16.5f\n',fval+mLikSpy);
fprintf('Number of observations:     %10.0f\n\n',size(g_mRK,1));
fprintf('Estimation period:          %9.8g - %8.8g\n',mtmp(stS,1),mtmp(enS,1));

elapsed = toc(time1); fprintf('Run time:  %15s\n', char(sec2time(elapsed)));

